<?php

namespace Drupal\Tests\layout_builder_sections_access\Functional;

use Drupal\layout_builder\Entity\LayoutBuilderEntityViewDisplay;
use Drupal\layout_builder\Plugin\SectionStorage\OverridesSectionStorage;
use Drupal\layout_builder\Section;
use Drupal\layout_builder\SectionComponent;
use Drupal\Tests\BrowserTestBase;
use Drupal\user\Entity\Role;

/**
 * Tests the access of layout sections.
 *
 * @group layout_builder_sections_access
 */
class SectionAccessTest extends BrowserTestBase {

  /**
   * {@inheritdoc}
   */
  protected static $modules = [
    'field_ui',
    'layout_builder',
    'layout_builder_sections_access',
    'node',
    'block_test',
  ];

  /**
   * {@inheritdoc}
   */
  protected $defaultTheme = 'starterkit_theme';


  /**
   * The user object.
   *
   * @var \Drupal\user\Entity\User
   */
  protected $user;

  /**
   * {@inheritdoc}
   */
  protected function setUp(): void {
    parent::setUp();

    $this->createContentType([
      'type' => 'bundle_with_section_field',
    ]);

    LayoutBuilderEntityViewDisplay::load('node.bundle_with_section_field.default')
      ->enableLayoutBuilder()
      ->setOverridable()
      ->save();

    $this->user = $this->drupalCreateUser([
      'configure any layout',
      'administer node display',
      'administer node fields',
      'administer content types',
    ], 'foobar');

    $this->drupalLogin($this->user);
  }

  /**
   * Test not-disabled sector.
   */
  public function testNotDisabledSection() {

    // Create layout data without disabled section.
    $layout_settings = [];
    $third_party_settings = [];
    $layout_data = $this->createLayoutData($layout_settings, $third_party_settings);

    // Create node.
    $node = $this->createSectionNode($layout_data);
    $canonical_url = $node->toUrl('canonical');

    // Get node and test.
    $this->drupalGet($canonical_url);
    $assert_session = $this->assertSession();
    $assert_session->pageTextContains('The node body');

  }

  /**
   * Test disabling sector.
   */
  public function testDisabledSection() {

    // Create layout data with disabled section.
    $layout_settings = [
      'layout_builder_sections_access_config' => [
        'disable_section' => 1,
        'visibility_roles' => [],
      ],
    ];
    $third_party_settings = [];
    $layout_data = $this->createLayoutData($layout_settings, $third_party_settings);

    // Create node.
    $node = $this->createSectionNode($layout_data);
    $canonical_url = $node->toUrl('canonical');

    // Get node and test.
    $this->drupalGet($canonical_url);
    $assert_session = $this->assertSession();
    $assert_session->pageTextNotContains('The node body');
  }

  /**
   * Test sector access by role.
   */
  public function testRoleAccessSection() {

    // Create new role.
    $role = Role::create([
      'id' => 'section_access',
      'label' => 'Role with section access',
    ]);

    // Create layout data with role access.
    $layout_settings = [
      'layout_builder_sections_access_config' => [
        'disable_section' => 0,
        'visibility_roles' => [
          $role->id(),
        ],
      ],
    ];
    $third_party_settings = [];
    $layout_data = $this->createLayoutData($layout_settings, $third_party_settings);

    // Create node.
    $node = $this->createSectionNode($layout_data);
    $canonical_url = $node->toUrl('canonical');

    // Get node and test.
    $this->drupalGet($canonical_url);
    $assert_session = $this->assertSession();
    $assert_session->pageTextNotContains('The node body');

    // Assign new role to user.
    $this->user->addRole($role->id());
    $this->user->save();

    // Get node and test.
    $this->drupalGet($canonical_url);
    $assert_session = $this->assertSession();
    $assert_session->pageTextContains('The node body');

  }

  /**
   * Build a section's layout data array.
   *
   * @param array $layout_settings
   *   An array of values for layout plugin settings.
   * @param array $third_party_settings
   *   An array of values for third party settings.
   *
   * @return array
   *   The layout data array.
   */
  protected function createLayoutData(array $layout_settings = [], array $third_party_settings = []) {
    $layout_data = [
      'section' => new Section(
        'layout_onecol',
        $layout_settings,
        [
          'baz' => new SectionComponent(
            'baz',
            'content', [
              'id' => 'field_block:node:bundle_with_section_field:body',
              'context_mapping' => [
                'entity' => 'layout_builder.entity',
              ],
            ]
          ),
        ],
        $third_party_settings,
      ),
    ];
    return $layout_data;
  }

  /**
   * Creates a node with a section field.
   *
   * @param array $section_values
   *   An array of values for a section field.
   *
   * @return \Drupal\node\NodeInterface
   *   The node object.
   */
  protected function createSectionNode(array $section_values) {
    return $this->createNode([
      'type' => 'bundle_with_section_field',
      'title' => 'The node title',
      'body' => [
        [
          'value' => 'The node body',
        ],
      ],
      OverridesSectionStorage::FIELD_NAME => $section_values,
    ]);
  }

}
